/**
 * MyAMS page initialization
 * This code is called once to register global events and callbacks
 */
(function($, globals) {

	var ams = globals.MyAMS;

	ams.initPage = function() {

		var body = $('body');

		/* Init main components */
		ams.root = body;
		ams.leftPanel = $('#left-panel');
		ams.shortcuts = $('#shortcuts');
		ams.plugins.initData(body);

		// Init main AJAX events
		var xhr = $.ajaxSettings.xhr;
		$.ajaxSetup({
			beforeSend: function(request, options) {
				// Check CSRF token for unsafe methods
				if (ams.safeMethods.indexOf(options.type) < 0) {
					if (globals.Cookies !== undefined) {
						var token = Cookies.get(ams.csrfCookieName);
						if (token) {
							request.setRequestHeader(ams.csrfHeaderName, token);
						}
					}
				}
			},
			progress: ams.ajax && ams.ajax.progress,
			progressUpload: ams.ajax && ams.ajax.progress,
			xhr: function() {
				var request = xhr();
				if (request && (typeof(request.addEventListener) === "function")) {
					var that = this;
					if (that && that.progress) {
						request.addEventListener("progress", function (evt) {
							that.progress(evt);
						}, false);
					}
				}
				return request;
			}
		});
		$(document).ajaxStart(ams.ajax && ams.ajax.start);
		$(document).ajaxStop(ams.ajax && ams.ajax.stop);
		$(document).ajaxError(ams.error && ams.error.ajax);

		// Check for minified state in local storage
		var state = globals.localStorage && globals.localStorage.getItem('window-state');
		if (state) {
			body.addClass(state);
		}

		// Check mobile/desktop
		if (!ams.isMobile) {
			ams.root.addClass('desktop-detected');
			ams.device = 'desktop';
		} else {
			ams.root.addClass('mobile-detected');
			ams.device = 'mobile';
			if (ams.enableFastclick) {
				ams.ajax && ams.ajax.check($.fn.noClickDelay,
										   ams.baseURL + '/ext/jquery-smartclick' + ams.devext + '.js',
										   function() {
											   $('NAV UL A').noClickDelay();
											   $('A', '#hide-menu').noClickDelay();
										   });
			}
		}

		// Switch shortcuts
		$('#show-shortcuts').click(function(e) {
			if (ams.shortcuts.is(":visible")) {
				ams.skin && ams.skin._hideShortcutButtons();
			} else {
				ams.skin && ams.skin._showShortcutButtons();
			}
			e.preventDefault();
		});
		ams.shortcuts.click(function(e) {
			ams.skin && ams.skin._hideShortcutButtons();
		});

		$(document).mouseup(function(e) {
			if (!ams.shortcuts.is(e.target) &&
				ams.shortcuts.has(e.target).length === 0) {
				ams.skin && ams.skin._hideShortcutButtons();
			}
		});

		// Show & hide mobile search field
		$('#search-mobile').click(function() {
			ams.root.addClass('search-mobile');
		});

		$('#cancel-search-js').click(function() {
			ams.root.removeClass('search-mobile');
		});

		// Activity badge
		$('.activity-button', '#user-activity').click(function(e) {
			var activity = $('#user-activity');
			var dropdown = $('.ajax-dropdown', activity);
			if (!dropdown.is(':visible')) {
				dropdown.css('left', - dropdown.innerWidth() + activity.innerWidth())
						.fadeIn(150);
				activity.addClass('active');
			} else {
				dropdown.fadeOut(150);
				activity.removeClass('active');
			}
			e.preventDefault();
		});
		ams.skin && ams.skin.checkNotification();

		$(document).mouseup(function(e) {
			var dropdown = $('.ajax-dropdown');
			if (!dropdown.is(e.target) &&
				dropdown.has(e.target).length === 0) {
				dropdown.fadeOut(150)
						.prev().removeClass("active");
			}
		});

		$('input[name="activity"]').change(function(e) {
			var href = $(this).data('ams-url');
			if (href) {
				e.preventDefault();
				e.stopPropagation();
				var hrefGetter = ams.getFunctionByName(href);
				if (typeof(hrefGetter) === 'function') {
					href = hrefGetter.call(this);
				}
				if (typeof(href) === 'function') {
					// Javascript function call
					href.call(this);
				} else {
					var container = $('.ajax-notifications');
					ams.skin && ams.skin.loadURL(href, container);
				}
			}
		});

		// Logout button
		$('a', '#user-menu LI.logout').click(function(e) {
			e.preventDefault();
			e.stopPropagation();
			//get the link
			ams.loginURL = $(this).attr('href');
			// ask verification
			ams.skin && ams.skin.bigBox({
				title : "<i class='fa fa-sign-out txt-color-orangeDark'></i> " + ams.i18n.LOGOUT +
						" <span class='txt-color-orangeDark'><strong>" + $('#show-shortcut').text() + "</strong></span> ?",
				content : ams.i18n.LOGOUT_COMMENT,
				buttons : ams.i18n.BTN_YES_NO
			}, function(ButtonPressed) {
				if (ButtonPressed === ams.i18n.BTN_YES) {
					ams.root.addClass('animated fadeOutUp');
					setTimeout(ams.skin.logout, 1000);
				}
			});
		});

		// Initialize left nav
		var nav = $('nav');
		$('UL', nav).myams_menu({
			accordion : nav.data('ams-menu-accordion') !== false,
			speed : ams.menuSpeed
		});

		// Left navigation hide button
		$('#hide-menu').find('>:first-child >A').click(function(e) {
			body.toggleClass("hidden-menu");
			if (globals.localStorage) {
				if (body.hasClass('hidden-menu')) {
					globals.localStorage.setItem('window-state', 'hidden-menu');
				} else {
					globals.localStorage.setItem('window-state', '');
				}
			}
			e.preventDefault();
		});

		// Left navigation collapser
		$('.minifyme').click(function(e) {
			body.toggleClass("minified");
			if (globals.localStorage) {
				if (body.hasClass('minified')) {
					globals.localStorage.setItem('window-state', 'minified');
				} else {
					globals.localStorage.setItem('window-state', '');
				}
			}
			$(this).effect("highlight", {}, 500);
			e.preventDefault();
		});

		// Reset widgets
		$('#refresh').click(function(e) {
			ams.skin && ams.skin.bigBox({
				title: "<i class='fa fa-refresh' style='color: green'></i> " + ams.i18n.CLEAR_STORAGE_TITLE,
				content: ams.i18n.CLEAR_STORAGE_CONTENT,
				buttons: '['+ams.i18n.BTN_CANCEL+']['+ams.i18n.BTN_OK+']'
			}, function(buttonPressed) {
				if (buttonPressed === ams.i18n.BTN_OK && localStorage) {
					localStorage.clear();
					location.reload();
				}
			});
			e.preventDefault();
		});

		// Check active pop-overs
		body.on('click', function(e) {
			var element = $(this);
			if (!element.is(e.target) &&
				element.has(e.target).length === 0 &&
				$('.popover').has(e.target).length === 0) {
				element.popover('hide');
			}
		});

		// Resize events
		ams.ajax && ams.ajax.check($.resize,
								   ams.baseURL + 'ext/jquery-resize' + ams.devext + '.js',
								   function() {
									   $('#main').resize(function() {
										   ams.skin._setPageHeight();
										   ams.skin._checkMobileWidth();
									   });
									   nav.resize(function() {
										   ams.skin._setPageHeight();
									   });
								   });

		// Init AJAX navigation
		if (ams.ajaxNav) {
			$(document).on('click', 'a[href="#"]', function(e) {
				e.preventDefault();
			});
			$(document).on('click', 'a[href!="#"]:not([data-toggle]), [data-ams-url]:not([data-toggle])', function(e) {
				var link = $(e.currentTarget);
				var handlers = link.data('ams-disabled-handlers');
				if ((handlers === true) || (handlers === 'click') || (handlers === 'all')) {
					return;
				}
				var href = link.attr('href') || link.data('ams-url');
				if (!href || href.startsWith('javascript') || link.attr('target') || (link.data('ams-context-menu') === true)) {
					return;
				}
				e.preventDefault();
				e.stopPropagation();

				var url,
					target,
					params;
				if (href.indexOf('?') >= 0) {
					url = href.split('?');
					target = url[0];
					params = url[1].unserialize();
				} else {
					target = href;
					params = undefined;
				}
				var hrefGetter = ams.getFunctionByName(target);
				if (typeof(hrefGetter) === 'function') {
					href = hrefGetter.call(link, params);
				}
				if (typeof(href) === 'function') {
					// Javascript function call
					href.call(link, params);
				} else {
					// Standard AJAX or browser URL call
					// Convert %23 chars to #
					href = href.replace(/\%23/, '#');
					if (e.ctrlKey) {
						window.open(href);
					} else {
						var link_target = link.data('ams-target');
						if (link_target) {
							if (link_target === '_blank') {
								window.open(href);
							} else {
								ams.form && ams.form.confirmChangedForm(link_target, function () {
									ams.skin && ams.skin.loadURL(href, link_target, link.data('ams-link-options'), link.data('ams-link-callback'));
								});
							}
						} else {
							ams.form && ams.form.confirmChangedForm(function() {
								if (href.startsWith('#')) {
									if (href !== location.hash) {
										if (ams.root.hasClass('mobile-view-activated')) {
											ams.root.removeClass('hidden-menu');
											window.setTimeout(function() {
												window.location.hash = href;
											}, 50);
										} else {
											window.location.hash = href;
										}
									}
								} else {
									window.location = href;
								}
							});
						}
					}
				}
			});
			$(document).on('click', 'a[target="_blank"]', function(e) {
				e.preventDefault();
				var target = $(e.currentTarget);
				window.open(target.attr('href'));
				ams.stats && ams.stats.logEvent(target.data('ams-stats-category') || 'Navigation',
												target.data('ams-stats-action') || 'External',
												target.data('ams-stats-label') || target.attr('href'));
			});
			$(document).on('click', 'a[target="_top"]', function(e) {
				e.preventDefault();
				ams.form && ams.form.confirmChangedForm(function() {
					window.location = $(e.currentTarget).attr('href');
				});
			});

			// Check URL when hash changed
			$(window).on('hashchange', ams.skin && ams.skin.checkURL);
		}

		// Initialize modal dialogs links
		$(document).off('click.modal')
				   .on('click', '[data-toggle="modal"]', function(e) {
			var source = $(this);
			var handlers = source.data('ams-disabled-handlers');
			if ((handlers === true) || (handlers === 'click') || (handlers === 'all')) {
				return;
			}
			if (source.data('ams-context-menu') === true) {
				return;
			}
			if (source.data('ams-stop-propagation') === true) {
				e.stopPropagation();
			}
			e.preventDefault();
			ams.dialog && ams.dialog.open(source);
			if (source.parents('#shortcut').exists()) {
				setTimeout(ams.skin._hideShortcutButtons, 300);
			}
		});

		// Initialize form buttons
		$(document).on('click', 'button[type="submit"], button.submit', function() {
			var button = $(this);
			$(button.get(0).form).data('ams-submit-button', button);
		});

		// Initialize main event handlers
		ams.initHandlers(document);

		// Submit form when CTRL+Enter key is pressed in textarea
		$(document).on('keydown', 'textarea', function(e) {
			if ((e.keyCode === 10 || e.keyCode === 13) && (e.ctrlKey || e.metaKey)) {
				$(this).closest('form').submit();
			}
		});

		// Handle update on file upload placeholder
		$(document).on('change', 'input[type="file"]', function(e) {
			e.preventDefault();
			var input = $(this);
			var button = input.parent('.button');
			if (button.exists() && button.parent().hasClass('input-file')) {
				button.next('input[type="text"]').val(input.val());
			}
		});

		// Always blur readonly inputs
		$(document).on('focus', 'input[readonly="readonly"]', function() {
			$(this).blur();
		});

		// Prevent bootstrap dialog from blocking TinyMCE focus
		$(document).on('focusin', function(e) {
			if ($(e.target).closest('.mce-window').length) {
				e.stopImmediatePropagation();
			}
		});

		// Disable clicks on disabled tabs
		$(document).on("click", '.nav-tabs a[data-toggle=tab]', function(e) {
			if ($(this).parent('li').hasClass("disabled")) {
				e.preventDefault();
				return false;
			}
		});

		// Automatically set orientation of dropdown menus
		$(document).on('show.bs.dropdown', '.btn-group', function() {
			var menu = $(this);
			var ul = menu.children('.dropdown-menu');
			var menuRect = menu.get(0).getBoundingClientRect();
			var position = menuRect.top;
			var buttonHeight = menuRect.height;
			var menuHeight = ul.outerHeight();
			if (position > menuHeight && $(window).height() - position < buttonHeight + menuHeight) {
				menu.addClass("dropup");
			}
		}).on('hidden.bs.dropdown', '.btn-group', function() {
			// always reset after close
			$(this).removeClass('dropup');
		});

		// Enable tabs dynamic loading
		$(document).on('show.bs.tab', function(e) {
			var link = $(e.target);
			if (link.exists() && (link.get(0).tagName !== 'A')) {
				link = $('a[href]', link);
			}
			var data = link.data();
			if (data && data.amsUrl) {
				if (data.amsTabLoaded) {
					return;
				}
				link.append('<i class="fa fa-spin fa-cog margin-left-5"></i>');
				ams.skin && ams.skin.loadURL(data.amsUrl, link.attr('href'), {
					afterLoadCallback: function() {
						if (data.amsTabLoadOnce) {
							link.data('ams-tab-loaded', true);
						}
						$('i', link).remove();
					},
					afterErrorCallback: function() {
						$('i', link).remove();
					}
				});
			}
		});

		// Check modal form dialogs on close
		$(document).on('hide.bs.modal', function(e) {
			var modal = $(e.target);
			ams.form && ams.form.confirmChangedForm(modal, function() {
				// Confirm closing if OK
				var bsModal = modal.data('modal') || modal.data('bs.modal');
				if (bsModal) {
					bsModal.isShown = true;
				}
				return true;
			}, function() {
				// Prevent closing if cancelled
				e.preventDefault();
				return false;
			});
		});

		// Enable custom MyAMS refresh events
		$(document).on('myams.refresh', function(event, options) {
			ams.executeFunctionByName(options.handler || (ams.skin && ams.skin.refreshContent), event.target, options);
		});

		// Init page content
		ams.initContent(document);
		if (ams.ajaxNav && nav.exists()) {
			ams.skin && ams.skin.checkURL();
		}
		ams.form && ams.form.setFocus(document);

		// Add unload event listener to check for modified forms
		$(window).on('beforeunload', ams.form && ams.form.checkBeforeUnload);

	};


	/**
	 * Main content plug-ins initializer
	 * This code is called to initialize plugins, callbacks and events listeners each time an HTML content
	 * is loaded dynamically from remote server.
	 */
	ams.initContent = function(element) {

		// Remove left tips
		$('.tipsy').remove();

		// Activate tooltips and popovers
		$("[rel=tooltip]", element).tooltip();
		$("[rel=popover]", element).popover();

		// Activate popovers with hover states
		$("[rel=popover-hover]", element).popover({
			trigger : "hover"
		});

		// Init registered plug-ins and callbacks
		ams.plugins && ams.plugins.init(element);
		ams.callbacks && ams.callbacks.init(element);
		ams.events && ams.events.init(element);
		ams.form && ams.form.init(element);

		// Initialize widgets
		if (ams.device === 'desktop') {
			ams.skin && ams.skin._initDesktopWidgets(element);
		} else {
			ams.skin && ams.skin._initMobileWidgets(element);
		}
		ams.skin && ams.skin._setPageHeight();

	};


	$(document).ready(function() {
		$ = jQuery.noConflict();
		var html = $('HTML');
		html.removeClass('no-js')
			.addClass('js');
		var lang = html.attr('lang') || html.attr('xml:lang');
		if (lang && !lang.startsWith('en')) {
			ams.lang = lang;
			ams.getScript(ams.baseURL + 'i18n/myams_' + lang.substr(0, 2) + ams.devext + '.js', function () {
				ams.initPage();
			});
		} else {
			ams.initPage();
		}
	});

})(jQuery, this);
